# -*- coding: utf-8 -*-

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'paymentcash',
    }
}

PAYMECASH_WALLET_ID = '000000001060'
PAYMECASH_DEFAULT_CURRENCY = 'RUB'
PAYMECASH_SECRET_KEY = '1d43a7561b5cb5af7877994ee1dc'
PAYMECASH_HIDE_FORM = False


# -*- coding: utf-8 -*-

import conf
from django.forms import model_to_dict
from .signals import payment_fail
from .signals import payment_process
from .signals import payment_completed
from django.db import models


class Payment(models.Model):
    PAYMENT_CARD = 1
    PATMENT_UKASH = 13
    PATMENT_WEBMONEY = 15
    PATMENT_YANDEX_MONEY = 16
    PATMENT_QIWI = 21
    PATMENT_MAILRU = 150
    PATMENT_PAYCASH = 300
    PATMENT_MONEBOOKERS = 410
    PAYMENT_MODES_CHOICES = (
        (PAYMENT_CARD, 'Банковские карты'),
        (PATMENT_UKASH, 'Ukash'),
        (PATMENT_WEBMONEY, 'Webmoney'),
        (PATMENT_YANDEX_MONEY, 'Яндекс-Деньги'),
        (PATMENT_QIWI, 'Qiwi'),
        (PATMENT_MAILRU, 'money@mail.ru'),
        (PATMENT_PAYCASH, 'Счет Paymecash'),
        (PATMENT_MONEBOOKERS, 'Счет Monebookers'),
    )

    CURRENCY_RUB = 'RUB'
    CURRENCY_USD = 'USD'
    CURRENCY_CHOICES = (
        (CURRENCY_RUB, 'Рубли'),
        (CURRENCY_USD, 'Доллары'),
    )

    RESULT_OK = 'OK'
    RESULT_ERROR = 'ERROR'
    RESULT_CHOICES = (
        (RESULT_OK, 'Подьвержден'),
        (RESULT_ERROR, 'Ошибка'),
    )

    order_id = models.CharField('Номер заказа', max_length=16, unique=True)
    wallet_id = models.PositiveIntegerField('Номер кошелька', default=conf.PAYMECASH_WALLET_ID)
    product_price = models.DecimalField('Сумма', max_digits=9, decimal_places=2)
    product_currency = models.CharField('Валюта', max_length=3,
                                        default=conf.PAYMECASH_DEFAULT_CURRENCY, choices=CURRENCY_CHOICES)
    payment_type_group_id = models.PositiveSmallIntegerField('Способ платежа', blank=True, null=True,
                                                             choices=PAYMENT_MODES_CHOICES, default=PAYMENT_CARD)
    cs1 = models.CharField('CS1', max_length=255, blank=True, null=True)
    cs2 = models.CharField('CS2', max_length=255, blank=True, null=True)
    cs3 = models.CharField('CS3', max_length=255, blank=True, null=True)

    result = models.CharField('Статус', max_length=16, choices=RESULT_CHOICES, blank=True, null=True)
    transaction_id = models.PositiveSmallIntegerField('Номер транзакции', blank=True, null=True)

    updated = models.DateTimeField('Обновлен', auto_now=True)
    created = models.DateTimeField('Создан', auto_now_add=True)

    def __unicode__(self):
        return '%s <%s>' % (
            self.order_id,
            self.get_status_display()
        )

    def save(self, *args, **kwargs):
        data = model_to_dict(self)
        if self.pk is None:
            payment_process.send(sender=self, data=data)
        else:
            if self.result == self.RESULT_OK:
                payment_completed.send(sender=self, data=data)
            if self.result == self.RESULT_ERROR:
                payment_fail.send(sender=self, data=data)
        super(Payment, self).save(*args, **kwargs)

    class Meta:
        ordering = ('created',)
        verbose_name = 'Платеж'
        verbose_name_plural = 'Платежи'


# -*- coding: utf-8 -*-

from django.contrib import admin
from .models import Payment


class PaymentAdmin(admin.ModelAdmin):
    list_display = ('order_id', 'result', 'wallet_id', 'product_price',
                    'product_currency', 'payment_type_group_id', 'transaction_id', 'created')
    list_filter = ('wallet_id', 'payment_type_group_id', 'result', 'created')
    search_fields = ('order_id', 'transaction_id', 'cs1', 'cs2', 'cs3')

admin.site.register(Payment, PaymentAdmin)


# -*- coding: utf-8 -*-

import sys
import os.path

try:
    from settings_local import *
except ImportError:
    print "Don't forget create settings_local.py"

ROOT = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, ROOT)
sys.path.append(os.path.join(ROOT, '..'))

DEBUG = True
TEMPLATE_DEBUG = DEBUG

SITE_ID = 1

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.debug',
    'django.core.context_processors.media',
    'django.core.context_processors.request',
    'django.contrib.messages.context_processors.messages',
)

SECRET_KEY = 'SECKEY'
ROOT_URLCONF = 'urls'
WSGI_APPLICATION = 'wsgi.application'

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.admin',
    'south',
    'factory',
    'cart',
    'paymecash',
)

SOUTH_TESTS_MIGRATE = False
SKIP_SOUTH_TESTS = True

INTERNAL_IPS = [
    'localhost',
    '127.0.0.1',
]

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

TEMPLATE_DIRS = (
    os.path.join(ROOT, 'templates'),
)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s %(funcName)s %(message)s'
        }
    },
    'handlers': {
        'paymecash': {
            'level': 'DEBUG',
            'class': 'logging.handlers.WatchedFileHandler',
            'filename': os.path.join(ROOT, 'paymecash.log'),
            'formatter': 'verbose'
        }
    },
    'loggers': {
        'paymecash': {
            'handlers': ['paymecash'],
            'level': 'ERROR',
            'propagate': True
        }
    }
}

try:
    from settings_local import *
except ImportError:
    pass


# -*- coding: utf-8 -*-

from django.contrib import admin
from .models import Goods
from .models import Order


class GoodsInLine(admin.StackedInline):
    extra = 0
    model = Goods

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class OrderAdmin(admin.ModelAdmin):
    list_display = ('order_id', 'amount')

    inlines = (GoodsInLine,)

admin.site.register(Order, OrderAdmin)


# -*- coding: utf-8 -*-

from django.views.generic import TemplateView
from paymecash.forms import PayForm
from cart.models import Goods
from cart.models import Order


class PayPageView(TemplateView):
    template_name = 'pay_page.html'

    def get_context_data(self, **kwargs):
        order = self.make_order()

        ctx = super(PayPageView, self).get_context_data(**kwargs)
        ctx['form'] = PayForm(initial={
            'order_id': order.order_id,
            'product_price': order.amount,
            'cs1': u'Описание заказа',
        })
        return ctx

    def make_order(self):
        item1, created = Goods.objects.get_or_create(name='Goods #1', price=100.)
        item2, created = Goods.objects.get_or_create(name='Goods #2', price=200.)

        order = Order(amount=item1.price + item2.price)
        order.save()
        order.goods.add(item1)
        order.goods.add(item2)
        order.save()

        return order


# -*- coding: utf-8 -*-

import uuid
import conf
import hashlib


def get_sign(data):
    parts = [str(int(data['wallet_id']))]
    if data.get('product_price') is not None and data.get('product_currency') is not None:
        parts.append(str(data['product_price']))
        parts.append(data['product_currency'])
    if data.get('order_id'):
        parts.append(data['order_id'])
    parts.append(data['secret_key'])
    string = '-'.join(parts)
    sign = hashlib.md5(string).hexdigest()
    return sign


def get_order_id():
    return str(uuid.uuid4()).replace('-', '')[:16]


# -*- coding: utf-8 -*-

import logging
from django.http import HttpResponse
from django.views.generic import View
from annoying.functions import get_object_or_None
from django.views.decorators.csrf import csrf_exempt
from .signals import payment_completed
from .signals import payment_fail
from .models import Payment

logger = logging.getLogger('paymecash')


class Confirm(View):
    @csrf_exempt
    def dispatch(self, *args, **kwargs):
        return super(Confirm, self).dispatch(*args, **kwargs)

    def post(self, request, *args, **kwargs):
        # TODO: здесь нужно проверять хэш данных, которого нет

        # Get payment
        order_id = request.POST.get('order_id')
        payment = get_object_or_None(Payment, order_id=order_id)
        if not payment:
            logger.error('Payment with order_id "%s" was not found' % order_id)
            payment_fail.send(sender=Payment, data=request.POST.copy())
            return HttpResponse()

        # Checking the error message and set if it exist
        message = request.POST.get('message')
        if message:
            payment.result = Payment.RESULT_ERROR
            logger.error('Recieved error message for order_id %s' % order_id)
            payment_fail.send(sender=Payment, data=request.POST.copy())
            return HttpResponse()

        transaction_id = request.POST.get('transaction')
        if transaction_id.isdigit():
            payment.transaction_id = request.POST.get('transaction')
        payment.result = Payment.RESULT_OK
        payment.save()
        payment_completed.send(sender=Payment, data=request.POST.copy())
        logger.info('Payment with order_id "%s" and transaction_id "%s" was confirmed' % (order_id, transaction_id))

        return HttpResponse()


# -*- coding: utf-8 -*-

from django.db import models
from paymecash.common import get_order_id


class Goods(models.Model):
    name = models.CharField(max_length=8)
    price = models.DecimalField(max_digits=9, decimal_places=2)


class Order(models.Model):
    order_id = models.CharField(max_length=16, default=get_order_id)
    amount = models.DecimalField(max_digits=9, decimal_places=2)
    goods = models.ManyToManyField(Goods)


# -*- coding: utf-8 -*-

from mock import patch
from django_webtest import WebTest
from paymecash.common import get_sign
from paymecash.common import get_order_id
from django.core.urlresolvers import reverse
from django.test.utils import override_settings
from .models import Payment


class CommonTests(WebTest):
    @override_settings(PAYMECASH_SECRET_KEY='1d43a7561b5cb5af7877994ee1dc')
    def test_get_sign(self):
        sign = get_sign({
            'product_price': 100.0,
            'wallet_id': '000000001060',
            'product_currency': 'RUB'
        })
        self.assertEqual(sign, '804ed09695d4dd1597d9eb1d9b876606', 'Sign is not equals')

    @patch('uuid.uuid4', lambda: '52bdd583-10da-4115-9479-883198df9f48')
    def test_get_order_id(self):
        order_id = get_order_id()
        self.assertEqual(order_id, '52bdd58310da4115', 'Order ID is not equals')


class PaymentTest(WebTest):
    def test_succes_confirm(self):
        payment = Payment(
            order_id='123',
            wallet_id=123,
            product_price=100.,
        )
        payment.save()

        data = {
            'transaction': '9120',
            'order_id': payment.order_id,
            'message': '',
        }
        url = reverse('paymentcash_confirm')
        self.app.post(url, params=data)

        self.assertEqual(payment.transaction_id, data['transaction'], 'Transaction ID was not set')
        self.assertEqual(payment.result, Payment.RESULT_OK, 'Result is not OK')

    def test_error_confirm(self):
        payment = Payment(
            order_id='123',
            wallet_id=123,
            product_price=100.,
        )
        payment.save()

        data = {
            'transaction': '9120',
            'order_id': 'fail_order',
            'message': '',
        }
        url = reverse('paymentcash_confirm')
        self.app.post(url, params=data)

        self.assertNotEqual(payment.transaction_id, data['transaction'], 'Transaction ID was set at fail order')
        self.assertEqual(payment.result, Payment.RESULT_ERROR, 'Result is not ERROR')


# -*- coding: utf-8 -*-

import conf
from django import forms
from .common import get_sign
from .models import Payment


class PayForm(forms.ModelForm):
    FIELDS_FOR_SIGN = ('wallet_id', 'product_price', 'product_currency', 'order_id')

    sign = forms.CharField(widget=forms.HiddenInput())

    def __init__(self, *args, **kwargs):
        super(PayForm, self).__init__(*args, **kwargs)

        data = {
            'wallet_id': self.fields['wallet_id'].initial,
            'product_price': self.fields['product_price'].initial,
            'product_currency': self.fields['product_currency'].initial,
            'order_id': self.fields['order_id'].initial,
            'secret_key': self.initial.get('secret_key') or conf.PAYMECASH_SECRET_KEY
        }
        data.update(self.initial)
        self.fields['sign'].initial = get_sign(data)

        if conf.PAYMECASH_HIDE_FORM:
            for name in self.fields:
                self.fields[name].widget = forms.HiddenInput()
        else:
            self.fields['wallet_id'].widget.attrs['readonly'] = True
            self.fields['product_price'].widget.attrs['readonly'] = True
            self.fields['order_id'].widget.attrs['readonly'] = True

    class Meta:
        model = Payment
        fields = ('wallet_id', 'order_id', 'product_price', 'product_currency',
                  'cs1', 'cs2', 'cs3', 'payment_type_group_id')


# -*- coding: utf-8 -*-

from django.conf import settings

PAYMECASH_WALLET_ID = getattr(settings, 'PAYMECASH_WALLET_ID')
PAYMECASH_DEFAULT_CURRENCY = getattr(settings, 'PAYMECASH_DEFAULT_CURRENCY', 'RUB')
PAYMECASH_SECRET_KEY = getattr(settings, 'PAYMECASH_SECRET_KEY')
PAYMECASH_PAYMENT_URL = getattr(settings, 'PAYMECASH_PAYMENT_URL', 'https://paymecash.me/api/payment')
PAYMECASH_HIDE_FORM = getattr(settings, 'PAYMECASH_HIDE_FORM', True)


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Payment'
        db.create_table(u'paymecash_payment', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('order_id', self.gf('django.db.models.fields.CharField')(unique=True, max_length=16)),
            ('wallet_id', self.gf('django.db.models.fields.PositiveIntegerField')(default='000000001060')),
            ('product_price', self.gf('django.db.models.fields.DecimalField')(max_digits=9, decimal_places=2)),
            ('product_currency', self.gf('django.db.models.fields.CharField')(default='RUB', max_length=3)),
            ('payment_type_group_id', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=1, null=True, blank=True)),
            ('cs1', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('cs2', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('cs3', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('result', self.gf('django.db.models.fields.CharField')(max_length=16, null=True, blank=True)),
            ('transaction_id', self.gf('django.db.models.fields.PositiveSmallIntegerField')(null=True, blank=True)),
            ('updated', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal(u'paymecash', ['Payment'])


    def backwards(self, orm):
        # Deleting model 'Payment'
        db.delete_table(u'paymecash_payment')


    models = {
        u'paymecash.payment': {
            'Meta': {'ordering': "('created',)", 'object_name': 'Payment'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'cs1': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'cs2': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'cs3': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'order_id': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '16'}),
            'payment_type_group_id': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '1', 'null': 'True', 'blank': 'True'}),
            'product_currency': ('django.db.models.fields.CharField', [], {'default': "'RUB'", 'max_length': '3'}),
            'product_price': ('django.db.models.fields.DecimalField', [], {'max_digits': '9', 'decimal_places': '2'}),
            'result': ('django.db.models.fields.CharField', [], {'max_length': '16', 'null': 'True', 'blank': 'True'}),
            'transaction_id': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'wallet_id': ('django.db.models.fields.PositiveIntegerField', [], {'default': "'000000001060'"})
        }
    }

    complete_apps = ['paymecash']


# -*- coding: utf-8 -*-

from django.conf.urls import patterns, include, url
from django.contrib import admin
from views import PayPageView

admin.autodiscover()

urlpatterns = patterns('',
    url(r'^$', PayPageView.as_view(), name='pay_page'),
    url(r'^paymecash/', include('paymecash.urls')),
    url(r'^admin/', include(admin.site.urls)),
)